module LiveQA
  module Plugins
    module Sidekiq
      ##
      # LiveQA \Plugins \Sidekiq \Server \Middleware
      #
      # Server middleware for sidekiq
      #
      class ServerMiddleware

        def call(_worker, job, _queue)
          LiveQA::Store.set(:session_tracker_id, job['liveqa_session_tracker_id'] || SecureRandom.uuid)
          LiveQA::Store.set(:server_software, "Sidekiq #{::Sidekiq::VERSION}")
          store_worker_data(job)
          yield
        ensure
          LiveQA::Store.clear!
        end

        private

        def store_worker_data(job)
          LiveQA::Store.set(
            :worker,
            name: 'sidekiq',
            version: ::Sidekiq::VERSION,
            queue: job['queue'],
            class: job['class'],
            id: job['jid'],
            created_at: convert_to_iso8601(job['created_at']),
            process_at: convert_to_iso8601(job['enqueued_at']),
            args: job['args'],
            retried: job['retry_count'].is_a?(Integer),
            retry_number: job['retry_count'],
            failed_at: convert_to_iso8601(job['failed_at'])
          )
        end

        def convert_to_iso8601(int_time)
          Time.at(int_time).utc.iso8601(3)
        rescue StandardError => _e
          nil
        end

      end
    end
  end
end
